==> mythical-player-1.2.2.tgz: package/js/mythical-player-full.js <==
import { render, html, nothing } from 'lit-html';
import { ifDefined } from 'lit-html/directives/if-defined.js';
import { classMap } from 'lit-html/directives/class-map.js';
import { useEffect, useState } from 'haunted';
import { v4 as uuid } from 'uuid';
import { createComponent } from './_lib/create-component.js';
import { setBackground, resetBackground } from './_lib/background';
import { Controls } from './modules/controls';
import { Playlist } from './modules/playlist';
import { Timeline } from './modules/timeline';
import { TrackInfo } from './modules/track-info';
import { Scrollable } from './modules/scrollable';
import { customEventConjurer } from 'custom-event-conjurer';
import { getCover } from './_lib/get-cover';
import styleMythicalPlayer from '../sass/type-full.scss';
import { setupMediaSession } from './_lib/setup-media-session';

const AMBIENT_AUTO = 'auto';
const AMBIENT_LIGHT = 'light';
const AMBIENT_DARK = 'dark';

function Component(element) {
  const {
    props,
    id,
    ambient: _ambient,
    accent,
    locale: _locale = 'en'
  } = element;
  const {
    playlist = {},
    list,
    useShadowDOM = true,
    style: _style = {}
  } = props;
  const { height: _styleHeight = '42rem', trackInfoHorizontal = 'right' } =
    _style;
  const mediaId = `media-${id}`;
  const listId = `list-${id}`;
  const playlistScrollId = `playlist-scroll-${id}`;

  function getMediaElement() {
    return useShadowDOM
      ? element.shadowRoot.getElementById(mediaId)
      : document.getElementById(mediaId);
  }

  const [literals, setLiterals] = useState(null);
  const [indexSelected, setIndexSelected] = useState(null);
  const [isPlaying, setIsPlaying] = useState(false);
  const [currentTime, setCurrentTime] = useState(0);
  const [totalTime, setTotalTime] = useState(0);
  const [prominentColors, setProminentColors] = useState('');
  const [hasProminentColors, setHasProminentColors] = useState(false);
  const [ambient, setAmbient] = useState(
    _ambient === AMBIENT_AUTO ? getAmbient() : _ambient
  );

  useEffect(() => {
    if (navigator.mediaSession && indexSelected !== null) {
      setupMediaSession({
        index: indexSelected,
        playlist,
        handleAudio: {
          stop: handleAudio.stop,
          play: handleAudio.play,
          pause: handleSessionPause,
          next: handleSessionNext,
          previous: handleSessionPrevious
        }
      });
    } else if (navigator.mediaSession) {
      navigator.mediaSession.metadata = null;
    }
  }, [indexSelected]);

  useEffect(() => {
    const darkModePreference = window.matchMedia(
      '(prefers-color-scheme: dark)'
    );
    darkModePreference.addEventListener('change', event => {
      _ambient === AMBIENT_AUTO &&
        setAmbient(event.matches ? AMBIENT_DARK : AMBIENT_LIGHT);
    });
    import(`./_i18n/${_locale}.json`, { with: { type: 'json' } }).then(data => {
      setLiterals(data.default);
    });
  }, []);

  useEffect(() => {
    setAmbient(_ambient);
  }, [_ambient]);

  function handleSessionPrevious() {
    handleAudio.setTrack({ index: indexSelected - 1, fromControls: true });
  }

  function handleSessionNext() {
    handleAudio.setTrack({ index: indexSelected + 1, fromControls: true });
  }

  function handleSessionPause() {
    getMediaElement().pause();
    if (navigator.mediaSession) {
      navigator.mediaSession.playbackState = 'paused';
    }
    setIsPlaying(false);
  }

  function handleSessionPlay() {
    getMediaElement().play();
    setIsPlaying(true);
    if (navigator.mediaSession) {
      navigator.mediaSession.playbackState = 'playing';
    }
  }

  function handleSessionStop() {
    if (navigator.mediaSession) {
      navigator.mediaSession.playbackState = 'none';
    }
    setIndexSelected(null);
    const media = getMediaElement();
    media.pause();
    media.currentTime = 0;
    setTotalTime(0);
    setHasProminentColors(false);
    if (navigator.mediaSession) {
      navigator.mediaSession.metadata = null;
    }
    resetBackground({ setProminentColors });
  }

  function handleScroll({ index }) {
    customEventConjurer({
      name: 'scrollToPlayingItem',
      detail: {
        selectedItem: element.shadowRoot
          .getElementById(listId)
          .querySelector(`[data-playlist-item="${index}"]`)
      },
      bubbles: true
    });
  }

  const handleAudio = {
    next: handleSessionNext,
    previous: handleSessionPrevious,
    setTrack({ index, fromControls = false }) {
      setIndexSelected(index);
      if (fromControls) {
        handleScroll({ index });
      }
      if (playlist[index]?.cover?.name) {
        setHasProminentColors(true);
        setBackground({
          coverUrl: getCover(playlist[index]?.cover),
          setProminentColors
        });
      } else {
        setHasProminentColors(false);
        resetBackground({ setProminentColors });
      }
      setTimeout(() => {
        this.playInitial();
      }, 0);
    },
    stop: handleSessionStop,
    play: handleSessionPlay,
    playInitial() {
      const mediaElement = getMediaElement();
      mediaElement.currentTime = 0;
      mediaElement.play().then(function () {
        setTotalTime(mediaElement.duration);
        if (navigator.mediaSession) {
          navigator.mediaSession.playbackState = 'playing';
        }
        setIsPlaying(true);
      });
    },
    pause: handleSessionPause
  };

  function handleAudioEnd() {
    if (playlist.length - 1 === indexSelected) {
      handleAudio.stop();
    } else {
      handleAudio.setTrack({ index: indexSelected + 1, fromControls: true });
    }
  }

  function handleTimeUpdate(ev) {
    setCurrentTime(ev.currentTarget.currentTime);
  }

  function getAmbient() {
    return window.matchMedia('(prefers-color-scheme: dark)').matches
      ? AMBIENT_DARK
      : AMBIENT_LIGHT;
  }

  const mainClasses = classMap({
    'mythical-player': true,
    'root-container': true,
    'has-gradient': hasProminentColors,
    [`accent-${accent}`]: true,
    [`ambient-${ambient}`]: true,
    [`mythical-player--track-info-${trackInfoHorizontal}`]: true
  });
  return literals
    ? html`
        <style>
          ${styleMythicalPlayer}
        </style>
        <div
          class="${mainClasses}"
          style="--player--main-height: ${_styleHeight}; ${prominentColors}"
        >
          <div class="mythical-player__inner">
            <div class="mythical-player__body">
              <div class="mythical-player__playlist">
                ${Scrollable({
                  extraClasses: 'scrollable--default',
                  eventScrollToPlayingItem: true,
                  id: playlistScrollId,
                  content: function () {
                    return Playlist({
                      playlist,
                      list,
                      indexSelected,
                      isPlaying,
                      handleAudio,
                      listId
                    });
                  }
                })}
              </div>
              <div class="mythical-player__track-info">
                ${TrackInfo({
                  indexSelected,
                  track: playlist[indexSelected],
                  literals: literals.trackInfo
                })}
              </div>
            </div>
            <div class="mythical-player__timeline">
              ${Timeline({ totalTime, currentTime, getMediaElement })}
            </div>
            <div class="mythical-player__controls">
              ${Controls({
                props: { isPlaying, playlist, indexSelected },
                handleAudio,
                literals: literals.controls
              })}
            </div>
          </div>
        </div>
        ${Number.isInteger(indexSelected)
          ? html`
              <audio
                src="${playlist[indexSelected]?.file}"
                id="${mediaId}"
                @ended="${handleAudioEnd}"
                @timeupdate="${handleTimeUpdate}"
              ></audio>
            `
          : nothing}
      `
    : html`Loading...`;
}

export function MythicalPlayerComponent(props) {
  const {
    id: _id = uuid(),
    extraClasses = null,
    useShadowDOM = true,
    style = {},
    locale = 'en'
  } = props;
  const { accent = 'default', ambient = AMBIENT_AUTO } = style;
  createComponent({
    name: 'mythical-player',
    fnComponent: Component,
    useShadowDOM,
    observedAttributes: ['id', 'accent', 'ambient', 'locale']
  });

  return html`
    <mythical-player
      id="mythical-player-${_id}"
      ambient="${ambient}"
      accent="${accent}"
      class="${ifDefined(extraClasses || undefined)}"
      locale="${locale}"
      .props="${props}"
    ></mythical-player>
  `;
}

export function MythicalPlayer({ placeholder, props }) {
  render(MythicalPlayerComponent(props), placeholder);
}

==> mythical-player-1.2.3.tgz: package/js/mythical-player-full.js <==
import { render, html, nothing } from 'lit-html';
import { ifDefined } from 'lit-html/directives/if-defined.js';
import { classMap } from 'lit-html/directives/class-map.js';
import { useEffect, useState } from 'haunted';
import { v4 as uuid } from 'uuid';
import { createComponent } from './_lib/create-component.js';
import { setBackground, resetBackground } from './_lib/background';
import { Controls } from './type-full/controls';
import { Playlist } from './type-full/playlist';
import { Timeline } from './type-full/timeline';
import { TrackInfo } from './type-full/track-info';
import { Scrollable } from './type-full/scrollable';
import { customEventConjurer } from 'custom-event-conjurer';
import { getCover } from './_lib/get-cover';
import styleMythicalPlayer from '../sass/type-full.scss';
import { setupMediaSession } from './_lib/setup-media-session';

const AMBIENT_AUTO = 'auto';
const AMBIENT_LIGHT = 'light';
const AMBIENT_DARK = 'dark';

function Component(element) {
  const {
    props,
    id,
    ambient: _ambient,
    accent,
    locale: _locale = 'en'
  } = element;
  const {
    playlist = {},
    list,
    useShadowDOM = true,
    style: _style = {}
  } = props;
  const { height: _styleHeight = '42rem', trackInfoHorizontal = 'right' } =
    _style;
  const mediaId = `media-${id}`;
  const listId = `list-${id}`;
  const playlistScrollId = `playlist-scroll-${id}`;

  function getMediaElement() {
    return useShadowDOM
      ? element.shadowRoot.getElementById(mediaId)
      : document.getElementById(mediaId);
  }

  const [literals, setLiterals] = useState(null);
  const [indexSelected, setIndexSelected] = useState(null);
  const [isPlaying, setIsPlaying] = useState(false);
  const [currentTime, setCurrentTime] = useState(0);
  const [totalTime, setTotalTime] = useState(0);
  const [prominentColors, setProminentColors] = useState('');
  const [hasProminentColors, setHasProminentColors] = useState(false);
  const [ambient, setAmbient] = useState(
    _ambient === AMBIENT_AUTO ? getAmbient() : _ambient
  );

  useEffect(() => {
    if (navigator.mediaSession && indexSelected !== null) {
      setupMediaSession({
        index: indexSelected,
        playlist,
        handleAudio: {
          stop: handleAudio.stop,
          play: handleAudio.play,
          pause: handleSessionPause,
          next: handleSessionNext,
          previous: handleSessionPrevious
        }
      });
    } else if (navigator.mediaSession) {
      navigator.mediaSession.metadata = null;
    }
  }, [indexSelected]);

  useEffect(() => {
    const darkModePreference = window.matchMedia(
      '(prefers-color-scheme: dark)'
    );
    darkModePreference.addEventListener('change', event => {
      _ambient === AMBIENT_AUTO &&
        setAmbient(event.matches ? AMBIENT_DARK : AMBIENT_LIGHT);
    });
    import(`./_i18n/${_locale}.json`, { with: { type: 'json' } }).then(data => {
      setLiterals(data.default);
    });
  }, []);

  useEffect(() => {
    setAmbient(_ambient);
  }, [_ambient]);

  function handleSessionPrevious() {
    handleAudio.setTrack({ index: indexSelected - 1, fromControls: true });
  }

  function handleSessionNext() {
    handleAudio.setTrack({ index: indexSelected + 1, fromControls: true });
  }

  function handleSessionPause() {
    getMediaElement().pause();
    if (navigator.mediaSession) {
      navigator.mediaSession.playbackState = 'paused';
    }
    setIsPlaying(false);
  }

  function handleSessionPlay() {
    getMediaElement().play();
    setIsPlaying(true);
    if (navigator.mediaSession) {
      navigator.mediaSession.playbackState = 'playing';
    }
  }

  function handleSessionStop() {
    if (navigator.mediaSession) {
      navigator.mediaSession.playbackState = 'none';
    }
    setIndexSelected(null);
    const media = getMediaElement();
    media.pause();
    media.currentTime = 0;
    setTotalTime(0);
    setHasProminentColors(false);
    if (navigator.mediaSession) {
      navigator.mediaSession.metadata = null;
    }
    resetBackground({ setProminentColors });
  }

  function handleScroll({ index }) {
    customEventConjurer({
      name: 'scrollToPlayingItem',
      detail: {
        selectedItem: element.shadowRoot
          .getElementById(listId)
          .querySelector(`[data-playlist-item="${index}"]`)
      },
      bubbles: true
    });
  }

  const handleAudio = {
    next: handleSessionNext,
    previous: handleSessionPrevious,
    setTrack({ index, fromControls = false }) {
      setIndexSelected(index);
      if (fromControls) {
        handleScroll({ index });
      }
      if (playlist[index]?.cover?.name) {
        setHasProminentColors(true);
        setBackground({
          coverUrl: getCover(playlist[index]?.cover),
          setProminentColors
        });
      } else {
        setHasProminentColors(false);
        resetBackground({ setProminentColors });
      }
      setTimeout(() => {
        this.playInitial();
      }, 0);
    },
    stop: handleSessionStop,
    play: handleSessionPlay,
    playInitial() {
      const mediaElement = getMediaElement();
      mediaElement.currentTime = 0;
      mediaElement.play().then(function () {
        setTotalTime(mediaElement.duration);
        if (navigator.mediaSession) {
          navigator.mediaSession.playbackState = 'playing';
        }
        setIsPlaying(true);
      });
    },
    pause: handleSessionPause
  };

  function handleAudioEnd() {
    if (playlist.length - 1 === indexSelected) {
      handleAudio.stop();
    } else {
      handleAudio.setTrack({ index: indexSelected + 1, fromControls: true });
    }
  }

  function handleTimeUpdate(ev) {
    setCurrentTime(ev.currentTarget.currentTime);
  }

  function getAmbient() {
    return window.matchMedia('(prefers-color-scheme: dark)').matches
      ? AMBIENT_DARK
      : AMBIENT_LIGHT;
  }

  const mainClasses = classMap({
    'mythical-player': true,
    'root-container': true,
    'has-gradient': hasProminentColors,
    [`accent-${accent}`]: true,
    [`ambient-${ambient}`]: true,
    [`mythical-player--track-info-${trackInfoHorizontal}`]: true
  });
  return literals
    ? html`
        <style>
          ${styleMythicalPlayer}
        </style>
        <div
          class="${mainClasses}"
          style="--player--main-height: ${_styleHeight}; ${prominentColors}"
        >
          <div class="mythical-player__inner">
            <div class="mythical-player__body">
              <div class="mythical-player__playlist">
                ${Scrollable({
                  extraClasses: 'scrollable--default',
                  eventScrollToPlayingItem: true,
                  id: playlistScrollId,
                  content: function () {
                    return Playlist({
                      playlist,
                      list,
                      indexSelected,
                      isPlaying,
                      handleAudio,
                      listId
                    });
                  }
                })}
              </div>
              <div class="mythical-player__track-info">
                ${TrackInfo({
                  indexSelected,
                  track: playlist[indexSelected],
                  literals: literals.trackInfo
                })}
              </div>
            </div>
            <div class="mythical-player__timeline">
              ${Timeline({ totalTime, currentTime, getMediaElement })}
            </div>
            <div class="mythical-player__controls">
              ${Controls({
                props: { isPlaying, playlist, indexSelected },
                handleAudio,
                literals: literals.controls
              })}
            </div>
          </div>
        </div>
        ${Number.isInteger(indexSelected)
          ? html`
              <audio
                src="${playlist[indexSelected]?.file}"
                id="${mediaId}"
                @ended="${handleAudioEnd}"
                @timeupdate="${handleTimeUpdate}"
              ></audio>
            `
          : nothing}
      `
    : html`Loading...`;
}

export function MythicalPlayerComponent(props) {
  const {
    id: _id = uuid(),
    extraClasses = null,
    useShadowDOM = true,
    style = {},
    locale = 'en'
  } = props;
  const { accent = 'default', ambient = AMBIENT_AUTO } = style;
  createComponent({
    name: 'mythical-player',
    fnComponent: Component,
    useShadowDOM,
    observedAttributes: ['id', 'accent', 'ambient', 'locale']
  });

  return html`
    <mythical-player
      id="mythical-player-${_id}"
      ambient="${ambient}"
      accent="${accent}"
      class="${ifDefined(extraClasses || undefined)}"
      locale="${locale}"
      .props="${props}"
    ></mythical-player>
  `;
}

export function MythicalPlayer({ placeholder, props }) {
  render(MythicalPlayerComponent(props), placeholder);
}
